const Controller = require('./controller');
const PubSub = require('pubsub-js');
const { config } = require('../server-state');
const { requester } = require('../main/context');
const { synchronize } = require('../main/synchronizer');

class ConfigurationController extends Controller {

    constructor(configPanel) {
        super();

        this._configPanel = configPanel;

        PubSub.subscribe('save-conf', () => { this.saveConfiguration(); });
        PubSub.subscribe('del-conf', (msg, value) => { this.deleteConfiguration(value); });
        PubSub.subscribe('restore-conf', (msg, value) => { this.restoreConfiguration(value); });
        PubSub.subscribe('export-conf', () => { this.exportConfiguration(); });
        PubSub.subscribe('import-conf', (msg, value) => { this.importConfiguration(value); });
    }

    /**
     * Import a configuration list stored in a string.
     */
    importConfiguration(fileData) {
        let cfgList = JSON.parse(fileData);

        requester.request('importConfiguration', cfgList, (data) => {
            this._configPanel.clearConfig();
            config.configurations = {};

            // Add each configuration to the panel.
            for(let cfg of data) {
                config.configurations[cfg.id] = cfg;
                this._configPanel.addConfig(cfg);
            }
        });
    }

    /**
     * Export a configuration list to a file.
     */
    exportConfiguration() {
        requester.request('exportConfiguration', 0, () => {});
    }

    /**
     * Delete the configuration with the given id.
     */
    deleteConfiguration(id) {
        delete config.configurations[id];
        requester.request('deleteConfiguration', id, () => {});
    }

    /**
     * Save the current state of vEXP and save it into a configuration.
     */
    saveConfiguration() {
        requester.request('saveConfiguration', 0, (data) => {
            config.configurations[data.id] = data;
            this._configPanel.addConfig(data);
        });
    }

    /**
     * Set the current vEXP state with these of the given configuration id.
     */
    restoreConfiguration(id) {
        requester.request('restoreConfiguration', id, () => {
            requester.request('init', 0, (data) => {
                synchronize(data);
            });
        });
    }
}

module.exports = ConfigurationController;