const Controller = require('./controller');
const { plane, peaks, sample, atoms, matrix } = require('../server-state');

class LatticeController extends Controller {
    constructor(lattice3D) {
        super();

        this._lattice3D = lattice3D;

        this._visibility = false;

        // Initialization.
        this.init();

        // Events handlers.
        PubSub.subscribe('change-peaks', () => { this.drawLattice(); });
        PubSub.subscribe('change-plane', () => { this.drawLattice(); });
        PubSub.subscribe('change-sample', () => { this.drawLattice(); });
        PubSub.subscribe('change-atoms', () => { this.drawLattice(); });
    }

    init() {
        this.drawLattice();
    }

    /**
     * Update the peaks lattice 3D view.
     */
    drawLattice() {
        this._lattice3D.redrawLattice(peaks, atoms , sample, plane, matrix.bMatrix, this._visibility);
    }

    /**
     * Activate or deactivate visibility of peaks that are not
     * in plane.
     */
    setVisibility(visibility) {
        this._visibility = visibility;
        this.drawLattice();
    }
}

module.exports = LatticeController;