const Controller = require('./controller');
const PubSub = require('pubsub-js');
const { sample, matrix, plane, atoms } = require('../server-state');
const { requester } = require('../main/context');
const THREE = require('three');


class SampleController extends Controller {
    constructor(samplePanel) {
        super();

        this._samplePanel = samplePanel;

        // Action handlers.
        PubSub.subscribe('update-sample', (msg, value) => { this.updateSample(value); });

        // Event handler.
        PubSub.subscribe('change-sample', () => { this.sampleChange(); });
        PubSub.subscribe('import-cif', (msg, value) => { this.importCIF(value); });

        PubSub.subscribe('sample-sync', (msg, value) => { this.syncSample(value); });

        // The unit cell must be redrawn in case of a new client with the server having already a space group configured from a previous session.
        //PubSub.publish('redraw-unitcell', { "spacegroup": sample.sg });
    }

    init() {
        this._samplePanel.setInputs(sample);
    }

    /**
     * Update sample in reponse to model update.
     */
    sampleChange() {
        this._samplePanel.setInputs(sample);

        PubSub.publish('redraw-unitcell', { "spacegroup": sample.sg });
    }

    
    /**
     * Update samples parameters in model.
     */
    updateSample(sampleParam) {
        requester.request('setSample', sampleParam, (data) => {
            // Update local sample state.
            sample.update(data.sample);

            // Update local matrix.
            matrix.update(data.matrix);

            // Update local plane.
            plane.update(data.plane);

            // Update views.
            PubSub.publish('change-sample', 0);
            PubSub.publish('redraw-unitcell', { "spacegroup": sample.sg });
        });
    }


    /**
     * Import sample parameters from a CIF.
     */
    importCIF(cifParam) {
        requester.request('importCif', cifParam, (data) => {

            // Update local sample state.
            sample.update(data.sample);

            // Update local matrix.
            matrix.update(data.matrix);

            // Update local plane.
            plane.update(data.plane);

            // Update local atoms.
            atoms.update(data.atoms);

            // Update views.
            PubSub.publish('change-sample', 0);
            PubSub.publish('change-atoms-conf', 0);
            PubSub.publish('update-atom-controller', 0);
            PubSub.publish('redraw-unitcell', { "spacegroup": sample.sg });
        });        
    }

    syncSample(value) {
        requester.request('syncSample', value, (data) => {});
    }
}

module.exports = SampleController;
