const fs = require('fs');
const path = require('path');
const homeDirectory = require('os').homedir();

/**
 * Class saving the multi layout configuration for an Electron instance in the directory $HOME/.vexp.
 */
class ElectronMultiLayoutSerializer {

    constructor() {
        // Create the vexp path if it does not exist.
        let vexpPath = path.join(homeDirectory, '.vexp');

        if (!fs.existsSync(vexpPath)) {
            fs.mkdirSync(vexpPath);
        }

        // Get the saved config.
        this._layoutConfigPath = path.join(vexpPath, 'layout-config.json');
    }

    read() {
        
        // Read the file.
        if (fs.existsSync(this._layoutConfigPath)) {
            console.log('Reading layout config file : ' + this._layoutConfigPath);
            return JSON.parse(fs.readFileSync(this._layoutConfigPath, 'utf8'));
        }

        return null;
    }

    write(state) {
        fs.writeFile(this._layoutConfigPath, state, () => {});
    }
}

module.exports = ElectronMultiLayoutSerializer;