const { requester, menuEvent } = require('./context');
const { synchronize } = require('./synchronizer');
const myLayout = require('./multi-layout');
const state = require('../server-state');

requester.onReady(() => {

    console.log('requester ready');

    requester.request('init', 0, (data) => {

        synchronize(data);
        
        const viewManager = require('../utils/view-manager');
        const ReciprocalView3D = require('../views/reciprocal/reciprocal-view-3d');
        const UnitCellView3D = require('../views/unitcell-view-3d');

        // Create the views.
        const reciprocalView3D = new ReciprocalView3D();
        const unitCellView3D = new UnitCellView3D();

        // The initialised event is emitted after the initial componentCreated event.
        // We must register the listener here because it seems that it is called synchronously in myLayout.init().
        myLayout.on('initialised', () => {

            // Initialise the views.
            viewManager.init();

            // Start the animation.
            animateScenes();
        });

        myLayout.init();

        if (state.mode.spyMode) {
            myLayout.root.contentItems[0].contentItems[1].toggleMaximise();
        }
        
        // Prevent right click open context menu while dragging a sphere
        document.addEventListener("contextmenu", function(e){
            e.preventDefault();
        }, false);
        
        menuEvent.on('resync', () => {
            requester.request('init', 0, (data) => {
                synchronize(data);
                reciprocalView3D.reset();
            });
        });

        // Animate function.
        function animateScenes() {
            requestAnimationFrame(animateScenes);

            // Render the views.
            viewManager.render();
        }

    });
});
