const viewManager = require('../../utils/view-manager');
const { expdata } = require('../../server-state');

class ExpDataPanelView {

    constructor() {

        let html = `
        <div class='container' style='width:100%' id='expDataPanel'>
            <div class='row'>
                <label class='col-sm-2'>Type</label>
                <select class='col-sm-10' id='sqwtype'></select>
            </div>
            <div class='row'>
                <label class='col-sm-2'>File</label>
                <input class='col-sm-10 form-control-file' type='file' id='expfile' accept='.dat,.sqw' />
            </div>
            <div class='row'>
                <label class='col-sm-2'>Formula</label>
                <input class='col-sm-10' type='text' value='S = exp(-q*q/0.05)' class='form-control' id='texformula'/>
            </div>
            <div class='row'>
                <label class='col-sm-3'>Tex. Width</label>
                <input class='col-sm-3' type='text' value='128' class='form-control' id='texwidth'/>
                <label class='col-sm-3'>Tex. Height</label>
                <input class='col-sm-3' type='text' value='128' class='form-control' id='texheight'/>
            </div>
            <div class='row'>
                <label class='col-sm-3'>Min. S(q,w)</label>
                <input class='col-sm-3' type='text' value='0' class='form-control' id='texminsqw'/>
        
                <label class='col-sm-3'>Max. S(q,w)</label>
                <input class='col-sm-3' type='text' value='1' class='form-control' id='texmaxsqw'/>
            </div>
            <div class='row'>
                <label class='col-sm-3'>E Slice</label>
                <input class='col-sm-3' type='text' value='0' class='form-control' id='texE'/>
        
                <label class='col-sm-3'>E Width</label>
                <input class='col-sm-3' type='text' value='0.01' class='form-control' id='texESigma'/>
            </div>
        </div>`;

        // Using lambda function does not work here.
        myLayout.registerComponent('S(q,w)', function(container, state) {
            container.getElement().html(html);
        });

        viewManager.addView(this);
    }

    init() {
        // Fill in sqw type combo box.
        $("#sqwtype").append($("<option/>").val("sqw-none").html("None"));
        $("#sqwtype").append($("<option/>").val("sqw-data").html("Experimental Data"));
        $("#sqwtype").append($("<option/>").val("sqw-formula").html("Formula"));

        // On file selection change -> emulate form keypress.
        $('#expfile').on('change', () => { this.newInputs(); });

        // Input change action
        $("#expDataPanel input[type='text']").keypress(() => {
            if (event.which == 13 || event.keyCode == 13 || event.which == 9 || event.keyCode == 9) {
                this.newInputs();
            }
        });

        // Selection change action
        $("#expDataPanel select").change(() => {
            this.newInputs();
        });

        this.setInputs({
            sqwtype: expdata.sqwtype, texformula: expdata.texformula,
            texwidth: expdata.texwidth, texheight: expdata.texheight,
            texminsqw: expdata.texminsqw, texmaxsqw: expdata.texmaxsqw,
            texE: expdata.texE, texESigma: expdata.texESigma
        });
    }



    /**
     * Set experiment data inputs value.
     */
    setInputs(value) {
        $('#sqwtype').val(value.sqwtype);
        $('#texformula').val(value.texformula);
        $('#texwidth').val(value.texwidth);
        $('#texheight').val(value.texheight);
        $('#texminsqw').val(value.texminsqw);
        $('#texmaxsqw').val(value.texmaxsqw);
        $('#texE').val(value.texE);
        $('#texESigma').val(value.texESigma);
    }

    /**
     * Sends the new inputs.
     * The sending can be done asynchronously since there is a file to read in case of experimental data.
     */
    newInputs() {

        let inputs = {
            sqwtype: $('#sqwtype').val(),
            texformula: $('#texformula').val(),
            texwidth: $('#texwidth').val(),
            texheight: $('#texheight').val(),
            texminsqw: $('#texminsqw').val(),
            texmaxsqw: $('#texmaxsqw').val(),
            texE: $('#texE').val(),
            texESigma: $('#texESigma').val(),
        };

        // If a file is provided
        if (inputs.sqwtype == "sqw-data") {
            const files = $("#expfile")[0].files;

            if (files.length > 0) {
                // Get the name of the file and not path that is only available with Electron.
                let file = files[0].name;
                if (file === null || file === "") {
                    return;
                }

                inputs.filename = file;
                
                let reader = new FileReader();
                reader.readAsText(files[0], "UTF-8");

                reader.onload = (event) => {
                    let dat = event.target.result;
                    dat = dat.split("\n");
                    inputs.filedata = dat;

                    // Publish the new inputs.
                    PubSub.publish('update-expdata', inputs);
                };
            }
        }
        else {
            // Publish the new inputs.
            PubSub.publish('update-expdata', inputs);
        }
    }

    render() {
    }
}

module.exports = ExpDataPanelView;
