const viewManager = require('../../utils/view-manager');
const xtal = require('../../../server/model/xtal/xtal');
const { sample, mode } = require('../../server-state');

class SamplePanelView {

    constructor() {
       
        let html = `
        <div class='container' style='width:100%'>
            <div class='row'>
                <label class='col-sm-3'>a (&#8491;)</label>
                <input type='text' class='col-sm-3' id='as'/>

                <label class='col-sm-3'>&alpha; (&#176;)</label>
                <input type='text' class='col-sm-3' id='aa'/>
            </div>
            <div class='row'>
                <label class='col-sm-3'>b (&#8491;)</label>
                <input type='text' class='col-sm-3' id='bs'/>

                <label class='col-sm-3'>&beta; (&#176;)</label>
                <input type='text' class='col-sm-3' id='bb'/>
            </div>
            <div class='row'>
                <label class='col-sm-3'>c (&#8491;)</label>
                <input type='text' class='col-sm-3' id='cs'/>

                <label class='col-sm-3'>&gamma; (&#176;)</label>
                <input type='text' class='col-sm-3' id='cc'/>
            </div>
            <div class='row'>
                <label class='col-sm-3'>Space Group</label>
                <select class='col-sm-6' id='sg'></select>
            </div>
            <div class='row'>
                <label class='col-sm-3'>CIF</label>
                <input class='col-sm-6' type='file' id='cif' accept='.cif' />
                <button class='col-sm-3 btn btn-sm btn-primary' id='sampleSync' type='button'>Synced</button>
            </div>
        </div>`;

        // Using lambda function does not work here.
        myLayout.registerComponent('Sample', function(container, state) {
            container.getElement().html(html);
        });

        viewManager.addView(this);
    }



    init() {

        $('#cif').on('change', () => { 
            this.importCIF();
        });
          
        // Update plan parameters action.
        $("#as, #bs, #cs, #aa, #bb, #cc").keypress((event) => {
            if (event.which == 13 || event.keyCode == 13) {
                PubSub.publish('update-sample', this.getInputs());
            }
        });

        // Sample sg list change.
        $("#sg").change(() => {
            PubSub.publish('update-sample', this.getInputs());
        });

        // Fill in space groups combo box.
        for (let sgid in xtal.spaceGroups) {
            let sg = xtal.spaceGroups[sgid];
            let sgname = sg.no == 0 ? "Unknown" : "No. " + sg.no + ": " + sg.id;
            $("#sg").append($("<option/>").val(sgid).html(sgname));
        }

        // Hide the button if not in spy mode.
        if (!mode.spyMode) {
            $("#sampleSync").hide();
        }

        $("#sampleSync").click(() => {
            // Change the sync.
            PubSub.publish('sample-sync', !sample.sync);
        });

        // Initialize inputs from model data.
        this.setInputs({
            as: sample.as, bs: sample.bs, cs: sample.cs,
            aa: sample.aa, bb: sample.bb, cc: sample.cc,
            sg: sample.sg, sync: sample.sync
        });
    }

    /**
     * Set samples inputs.
     */
    setInputs(value) {
        $('#as').val(value.as);
        $('#as').prop('disabled', sample.fixedValues.includes('as'));

        $('#bs').val(value.bs);
        $('#bs').prop('disabled', sample.fixedValues.includes('bs'));

        $('#cs').val(value.cs);
        $('#cs').prop('disabled', sample.fixedValues.includes('cs'));

        $('#aa').val(value.aa);
        $('#aa').prop('disabled', sample.fixedValues.includes('aa'));

        $('#bb').val(value.bb);
        $('#bb').prop('disabled', sample.fixedValues.includes('bb'));

        $('#cc').val(value.cc);
        $('#cc').prop('disabled', sample.fixedValues.includes('cc'));

        $('#sg').val(value.sg);

        // Change the text.
        if (value.sync) {
            $("#sampleSync").text('Synced');

            // We remove the selected file from the view.
            $("#cif").val('');
        }
        else {
            $("#sampleSync").text('Sync');
        }
    }

    /**
     * Get samples inputs and store them into an object.
     */
    getInputs() {
        return {
            as: parseFloat($('#as').val()),
            bs: parseFloat($('#bs').val()),
            cs: parseFloat($('#cs').val()),
            aa: parseFloat($('#aa').val()),
            bb: parseFloat($('#bb').val()),
            cc: parseFloat($('#cc').val()),
            sg: $('#sg').val()
        }
    }


    /**
     * Import a CIF
     */
    importCIF() {
        let inputs = { };

        // If a file is provided
        const files = $("#cif")[0].files;

        if (files.length > 0) {
            // Get the name of the file and not path that is only available with Electron.
            let file = files[0].name;
            if (file === null || file === "") {
                return;
            }

            inputs.filename = file;
            
            let reader = new FileReader();
            reader.readAsText(files[0], "UTF-8");

            reader.onload = (event) => {
                let dat = event.target.result;
                inputs.filedata = dat;

                // Publish the new inputs.
                PubSub.publish('import-cif', inputs);
            };
        }
    }

    render() {
    }
}


module.exports = SamplePanelView;
