/*
 * Custom main menu. Refers to https://www.christianengvall.se/electron-menu/.
 *
 * Documentation on dialogs can be found at https://electron.atom.io/docs/api/dialog/.
 *
 * It is possible to redirect to an external url:
 * require('electron').shell.openExternal('https://electron.atom.io');
 */
const electron = require('electron');
const {app, Menu} = electron;
let togglePeaksInPlane = false;
let toggle2DView = false;
let toggleVRView = false;
let toggleEnergyView = false;

const template = [

    {
        label: 'File',
        submenu: [
            {
                label: 'Load'
            },
            {
                label: 'Save'
            },
        ]
    },


    {
        label: 'Instrument',
        submenu: [
            {
                label: 'Import'
            },
            {
                label: 'Details'
            },
            {
                label: 'Area'
            }
        ]
    },


    {
        label: 'Modes',
        submenu: [
            {
                label: 'Snap Mode',
                accelerator: 'Ctrl+S',
                type: 'radio',
                checked: false,
                click(item, focusedWindow) {
                    if (focusedWindow) focusedWindow.webContents.send('change-mode', 'snap');
                }
            },
            {
                label: 'Free Mode',
                accelerator: 'Ctrl+F',
                type: 'radio',
                checked: true,
                click(item, focusedWindow) {
                    if (focusedWindow) focusedWindow.webContents.send('change-mode', 'free');
                }
            }
        ]
    },


    {
        label: 'Reciprocal View 3D',
        submenu: [
            {
                label: 'Toggle Peaks In Plane',
                click (item, focusedWindow) {
                    togglePeaksInPlane = !togglePeaksInPlane;
                    if (focusedWindow) focusedWindow.webContents.send('toggle-peaks-in-plane', togglePeaksInPlane);
                }
            },
            {
                label: 'Toggle 2D View',
                click (item, focusedWindow) {
                    toggle2DView = !toggle2DView;
                    if (focusedWindow) focusedWindow.webContents.send('toggle-2d-view', toggle2DView);
                }
            },
            {
                label: 'Toggle Energy View',
                click (item, focusedWindow) {
                    toggleEnergyView = !toggleEnergyView;
                    if (focusedWindow) focusedWindow.webContents.send('toggle-energy-view', toggleEnergyView);
                }
            },
            {
                label: 'Toggle VR View',
                click(item, focusedWindow) {
                    toggleVRView = !toggleVRView;
                    if (focusedWindow) focusedWindow.webContents.send('toggle-vr-view', toggleVRView);
                }
            }
        ]
    },

    {
        label: 'View',
        submenu: [
            {
                label: 'Reload',
                accelerator: process.platform === 'darwin' ? 'Alt+Command+R' : 'Ctrl+Shift+R',
                click (item, focusedWindow) {
                    if (focusedWindow) focusedWindow.reload();
                }
            },
            {
                label: 'Toggle Developer Tools',
                accelerator: process.platform === 'darwin' ? 'Alt+Command+I' : 'Ctrl+Shift+I',
                click (item, focusedWindow) {
                    if (focusedWindow) focusedWindow.webContents.toggleDevTools()
                }
            },
            {
                role: 'togglefullscreen'
            },
            {
                label: 'Resync with Nomad',
                accelerator: 'CmdOrCtrl+R',
                click(item, focusedWindow) {
                    if (focusedWindow) focusedWindow.webContents.send('resync', 0);
                }
            }
        ]
    },


    {
        role: 'help',
        submenu: [
        {
            label: 'About',
            click () {
                // Open a dialog box.
                const dialog = electron.dialog;

                // This code is executing in the main process so that accessing to the application version is different from the renderer process.
                dialog.showMessageBox(
                    {
                        type: 'info',
                        buttons: ['Close'],
                        title: 'About vEXP',
                        message: 'Version ' + app.getVersion()
                    });
            }
        }
    ]
  }
]


if (process.platform === 'darwin') {
    const name = 'vEXP'
    template.unshift({
        label: name,
        submenu: [
            {
                role: 'about'
            },
            {
                type: 'separator'
            },
            {
                role: 'services',
                submenu: []
            },
            {
                type: 'separator'
            },
            {
                role: 'hide'
            },
            {
                role: 'hideothers'
            },
            {
                role: 'unhide'
            },
            {
                type: 'separator'
            },
            {
                role: 'quit'
            }
        ]
    })
}

Menu.setApplicationMenu(Menu.buildFromTemplate(template))
